﻿using CallCenter.Share.Requests;
using CallCenter.Users;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace CallCenter.Caches;

public class UserCacheManager : IUserCacheManager, IScopeDependency
{
    private readonly ITypedCache<Work> _cacheWork;
    private readonly IWorkRepository _workRepository;
    private readonly ITypedCache<List<Work>> _cacheWorks;
    private const string WorkKey = "Works";
    public UserCacheManager(
        ITypedCache<Work> cacheWork,
        IWorkRepository workRepository,
        ITypedCache<List<Work>> cacheWorks)
    {
        _cacheWork = cacheWork;
        _workRepository = workRepository;
        _cacheWorks = cacheWorks;
    }

    /// <summary>
    /// 查询用户当前工作记录
    /// </summary>
    /// <param name="userId"></param>
    /// <returns></returns>
    public Work GetWorkByUser(string userId)
    {
        var work = _cacheWork.GetOrSet(Work.GetKey(KeyMode.UserId, userId), k =>
        {
            var dbWork = _workRepository.GetCurrentWorkByUserAsync(userId)
                .GetAwaiter().GetResult();
            if (dbWork == null)
                throw new UserFriendlyException("该用户暂无工作记录");
            return dbWork;
        });
        return work;
    }

    /// <summary>
    /// 查询分机当前工作记录
    /// </summary>
    /// <param name="telNo"></param>
    /// <returns></returns>
    /// <exception cref="UserFriendlyException"></exception>
    public Work GetWorkByTel(string telNo)
    {
        var work = _cacheWork.GetOrSet(Work.GetKey(KeyMode.TelNo, telNo), k =>
        {
            var dbWork = _workRepository.GetCurrentWorkByTelAsync(telNo)
                .GetAwaiter().GetResult();
            if (dbWork == null)
                throw new UserFriendlyException("该分机暂无工作记录");
            return dbWork;
        });
        return work;
    }

    /// <summary>
    /// 查询所有在工作的记录
    /// </summary>
    /// <returns></returns>
    public List<Work>? GetWorks()
    {
        var works = _cacheWorks.GetOrSet(WorkKey, k =>
        {
            var works = _workRepository.QueryAsync(x => x.EndTime == null).GetAwaiter().GetResult();
            return works;
        });
        return works;
    }

    public void AddWorksCache(Work work)
    {
        var list = GetWorks();
        if (list == null)
        {
            list = new List<Work>();
        }
        list.Add(work);
        _cacheWorks.Set(WorkKey, list);
    }

    public void UpdateWorksCache(Work work)
    {
        var list = GetWorks();
        var model = list.First(x => x.Id == work.Id);
        if (model!=null)
        {
            list.Remove(model);
        }
        list.Add(work);
        _cacheWorks.Set(WorkKey, list);
    }


    public void RemoveWorksCache(string id)
    {
        var list = _cacheWorks.Get(WorkKey)?.ToList();
        if (list!=null)
        {
            var model = list.FirstOrDefault(x => x.Id == id);
            if (model!=null)
            {
                list.Remove(model);
                _cacheWorks.Set(WorkKey, list);
            }
        }
    }



    /// <summary>
    /// 查询分机是否处于工作
    /// </summary>
    /// <param name="telNo"></param>
    /// <returns></returns>
    public async Task<bool> IsWorkingByTelAsync(string telNo, CancellationToken cancellationToken)
    {
        var work = _cacheWork.Get(Work.GetKey(KeyMode.TelNo, telNo));

        if (work is null)
        {
            var dbWork = await _workRepository.GetCurrentWorkByTelAsync(telNo, cancellationToken);
            if (dbWork != null)
            {
                _cacheWork.Set(Work.GetKey(KeyMode.TelNo, telNo), dbWork);
                return true;
            }
            return false;
        }
        return true;
    }

    /// <summary>
    /// 查询分机是否处于工作
    /// </summary>
    /// <param name="userId"></param>
    /// <param name="cancellationToken"></param>
    /// <returns></returns>
    public async Task<bool> IsWorkingByUserAsync(string userId, CancellationToken cancellationToken)
    {
        //return _cacheWork.TryGetOrAdd(Work.GetKey(KeyMode.UserId, userId),
        //    k => _workRepository.GetCurrentWorkByUserAsync(userId).GetAwaiter().GetResult(),
        //    out _);
        var work = _cacheWork.Get(Work.GetKey(KeyMode.UserId, userId));
        if (work is null)
        {
            var dbWork = await _workRepository.GetCurrentWorkByUserAsync(userId, cancellationToken);
            if (dbWork is null) return false;
            _cacheWork.Set(Work.GetKey(KeyMode.UserId, userId), dbWork);
        }

        return true;
    }

    /// <summary>
    /// 根据用户更新工作记录
    /// </summary>
    /// <param name="work"></param>
    /// <returns></returns>
    public void UpdateWorkByUser(Work work)
    {
        _cacheWork.Set(Work.GetKey(KeyMode.UserId, work.UserId), work);
        _cacheWork.Set(Work.GetKey(KeyMode.TelNo, work.TelNo), work);
    }

    

}